@@ -6,10 +6,12 @@ from __future__ import with_statement
 
 import tests
 
+import os
 import gtk
 
 from zim.errors import Error
-from zim.notebook import get_notebook_list, Path, NotebookInfo
+from zim.notebook import get_notebook_list, Path, Page, NotebookInfo
+from zim.formats import ParseTree
 from zim.fs import File, Dir
 from zim.config import config_file
 from zim.gui.clipboard import Clipboard
@@ -162,12 +164,12 @@ class TestDialogs(tests.TestCase):
 		dialog = zim.gui.RenamePageDialog(self.ui, path=Path('Test:foo:bar'))
 		self.assertTrue(dialog.form['update'])
 		self.assertTrue(dialog.form.widgets['update'].get_property('sensitive'))
-		self.assertTrue(dialog.form['head'])
+		self.assertFalse(dialog.form['head']) # There is no heading
 		self.assertTrue(dialog.form.widgets['head'].get_property('sensitive'))
 		dialog.form['name'] = 'New'
 		dialog.assert_response_ok()
 		self.assertEqual(self.ui.mock_calls[-1],
-			('do_rename_page', Path('Test:foo:bar'), 'New', True, True))
+			('do_rename_page', Path('Test:foo:bar'), 'New', False, True))
 
 		dialog = zim.gui.RenamePageDialog(self.ui, path=Path('New:bar'))
 		self.assertFalse(dialog.form['update'])
@@ -179,6 +181,20 @@ class TestDialogs(tests.TestCase):
 		self.assertEqual(self.ui.mock_calls[-1],
 			('do_rename_page', Path('New:bar'), 'New', False, False))
 
+	def testRenamePageDialogWithHeadingChanges(self):
+		'''Test RenamePageDialog's heading auto-change option depending on
+		whether we have a changed heading or not.
+		'''
+		tree = ParseTree().fromstring('<zim-tree></zim-tree>')
+		tree.set_heading("bar")
+		self.ui.page = Page(Path("Test:foo:bar"), parsetree=tree)
+		self.ui.notebook.get_page = lambda path: self.ui.page
+		dialog = zim.gui.RenamePageDialog(self.ui, path=Path("Test:foo:bar"))
+		self.assertTrue(dialog.form['head'])
+		tree.set_heading("different")
+		dialog = zim.gui.RenamePageDialog(self.ui, path=Path("Test:foo:bar"))
+		self.assertFalse(dialog.form['head'])
+	
 	def testDeletePageDialog(self):
 		'''Test DeletePageDialog'''
 		# just check inputs are OK - skip output
@@ -224,21 +240,6 @@ class TestDialogs(tests.TestCase):
 		col = dialog.results_treeview.get_column(0)
 		dialog.results_treeview.row_activated((0,), col)
 
-	def testNewApplicationDialog(self):
-		'''Test NewApplicationDialog'''
-		from zim.gui.applications import NewApplicationDialog
-		dialog = NewApplicationDialog(self.ui, mimetype='text/plain')
-		dialog.form['name'] = 'Foo'
-		dialog.form['exec'] = 'foo %f'
-		app = dialog.assert_response_ok()
-		self.assertEqual(app.name, 'Foo')
-
-		dialog = NewApplicationDialog(self.ui, type='web_browser')
-		dialog.form['name'] = 'Foo'
-		dialog.form['exec'] = 'foo %f'
-		app = dialog.assert_response_ok()
-		self.assertEqual(app.name, 'Foo')
-
 	def testCustomToolDialog(self):
 		'''Test CustomTool dialogs'''
 		from zim.gui.customtools import CustomToolManagerDialog
@@ -335,7 +336,9 @@ class TestDialogs(tests.TestCase):
 		gui = zim.gui.GtkInterface()
 		gui.register_preferences('GtkInterface', zim.gui.ui_preferences)
 		gui.register_preferences('PageView', zim.gui.pageview.ui_preferences)
-		gui.load_plugins()
+		with FilterFailedToLoadPlugin():
+			# may miss dependencies for e.g. versioncontrol
+			gui.load_plugins()
 		self.ui.preferences_register = gui.preferences_register
 		self.ui.preferences = gui.preferences
 		self.ui.plugins = gui.plugins
@@ -375,6 +378,11 @@ class TestDialogs(tests.TestCase):
 		dialog = PluginConfigureDialog(pref_dialog, klass)
 		dialog.assert_response_ok()
 
+	def testTemplateEditorDialog(self):
+		from zim.gui.templateeditordialog import TemplateEditorDialog
+		dialog = TemplateEditorDialog(self.ui)
+		# TODO what to test here ??
+		dialog.assert_response_ok()
 
 	# Test for ExportDialog can be found in test/export.py
 	# Test for NotebookDialog is in separate class below
@@ -387,11 +395,19 @@ class FilterNoSuchImageWarning(tests.LoggingFilter):
 	message = 'No such image:'
 
 
+class FilterFailedToLoadPlugin(tests.LoggingFilter):
+
+	logger = 'zim'
+	message = 'Failed to load plugin'
+
+
 @tests.slowTest
 class TestGtkInterface(tests.TestCase):
 
 	def setUp(self):
-		self.ui = setupGtkInterface(self)
+		with FilterFailedToLoadPlugin():
+			# may miss dependencies for e.g. versioncontrol
+			self.ui = setupGtkInterface(self)
 
 	def tearDown(self):
 		self.ui.close()
@@ -426,7 +442,7 @@ class TestGtkInterface(tests.TestCase):
 		ui.open_notebook(nb)
 
 		# remove plugins
-		self.assertGreater(len(ui.plugins), 3) # default plugins
+		self.assertGreaterEqual(len(ui.plugins), 3) # default plugins without dependencies
 		plugins = [p.plugin_key for p in ui.plugins]
 		for name in plugins:
 			ui.unload_plugin(name)
@@ -435,7 +451,7 @@ class TestGtkInterface(tests.TestCase):
 		# and add them again
 		for name in plugins:
 			ui.load_plugin(name)
-		self.assertGreater(len(ui.plugins), 3)
+		self.assertGreaterEqual(len(ui.plugins), 3) # default plugins without dependencies
 
 		# check registering an URL handler
 		func = tests.Counter(True)
@@ -448,7 +464,6 @@ class TestGtkInterface(tests.TestCase):
 		'''Test main window'''
 		path = Path('Test:foo:bar')
 		window = self.ui.mainwindow
-		#~ print 'UISTATE INIT:', window.uistate
 
 		self.assertTrue(window.uistate['show_menubar'])
 		window.toggle_menubar()
@@ -468,19 +483,19 @@ class TestGtkInterface(tests.TestCase):
 		window.toggle_statusbar()
 		self.assertTrue(window.uistate['show_statusbar'])
 
-		self.assertTrue(window.uistate['show_sidepane'])
-		window.toggle_sidepane()
-		self.assertFalse(window.uistate['show_sidepane'])
-		window.toggle_sidepane()
-		self.assertTrue(window.uistate['show_sidepane'])
+		self.assertTrue(window.uistate['left_pane'][0])
+		window.toggle_panes()
+		self.assertFalse(window.uistate['left_pane'][0])
+		window.toggle_panes()
+		self.assertTrue(window.uistate['left_pane'][0])
 
-		# note: focus starts at sidepane due to toggle_sidepane above
+		# note: focus starts at sidepane due to toggle_panes above
 		self.assertEqual(window.get_focus(), window.pageindex.treeview)
 		self.assertEqual(window.get_selected_path(), path)
-		window.toggle_focus_sidepane()
+		window.toggle_sidepane_focus()
 		self.assertEqual(window.get_focus(), window.pageview.view)
 		self.assertEqual(window.get_selected_path(), path)
-		window.toggle_focus_sidepane()
+		window.toggle_sidepane_focus()
 		self.assertEqual(window.get_focus(), window.pageindex.treeview)
 		# TODO also check this with "show_sidepane" off
 
@@ -574,7 +589,32 @@ class TestGtkInterface(tests.TestCase):
 		self.ui.save_page()
 		self.assertFalse(self.ui.page.get_parsetree() is None)
 
-	# TODO notebook manipulation (new (sub)page, move, rename, delete ..)
+	def testPageMove(self):
+		oldpath, newpath = Path('Movers:Stator:Mover'), Path('Movers:Mover')
+
+		# Open page and process message queue to sync tree view
+		indexpath = self.ui.notebook.index.lookup_path(oldpath)
+		self.ui.open_page(indexpath)
+		while gtk.events_pending():
+			gtk.main_iteration(False)
+
+		# Test actual moving
+		page = self.ui.notebook.get_page(oldpath)
+		text = page.dump('wiki')
+		self.ui.notebook.index.ensure_update()
+		self.ui.notebook.move_page(oldpath, newpath)
+		self.ui.notebook.index.ensure_update()
+
+		# newpath should exist and look like the old one
+		page = self.ui.notebook.get_page(newpath)
+		self.assertEqual(page.dump('wiki'), text)
+
+		# oldpath should be deleted
+		page = self.ui.notebook.get_page(oldpath)
+		self.assertFalse(page.haschildren)
+		self.assertFalse(page.hascontent)
+
+	# TODO notebook manipulation (new (sub)page, rename, delete ..)
 	# merge with tests for dialogs (?)
 
 	def testClipboard(self):
@@ -595,14 +635,19 @@ class TestClickLink(tests.TestCase):
 				zim.gui.GtkInterface.__init__(self, *arg, **kwarg)
 				tests.MockObjectBase.__init__(self)
 				for method in (
-					'open_file',
-					'open_with',
 					'open_notebook',
 					'open_page',
+					'open_file',
+					'_open_with_emailclient',
+					'_open_with_webbrowser',
+					'_open_with_filebrowser',
+					'_open_with',
 				):
 					self.mock_method(method, None)
 
-		self.ui = setupGtkInterface(self, klass=MyMock)
+		with FilterFailedToLoadPlugin():
+			# may miss dependencies for e.g. versioncontrol
+			self.ui = setupGtkInterface(self, klass=MyMock)
 
 	def tearDown(self):
 		self.ui.close()
@@ -634,9 +679,10 @@ class TestClickLink(tests.TestCase):
 			('foo', 'page'),
 			('foo:bar', 'page'),
 		):
+			#~ print ">> LINK %s (%s)" % (href, type)
 			#~ self.ui.open_url(href)
 			self.ui.mainwindow.pageview.do_link_clicked({'href': href})
-			msg = "Clicked: %s\nResulted in: %s" % (href, self.ui.mock_calls[-1])
+			msg = "Clicked: \"%s\" resulted in: \"%s\"" % (href, self.ui.mock_calls[-1])
 			if type == 'notebook':
 				self.assertTrue(self.ui.mock_calls[-1][0] == 'open_notebook', msg=msg)
 			elif type == 'page':
@@ -644,9 +690,11 @@ class TestClickLink(tests.TestCase):
 			elif type == 'file':
 				self.assertTrue(self.ui.mock_calls[-1][0] == 'open_file', msg=msg)
 			elif type == 'mailto':
-				self.assertTrue(self.ui.mock_calls[-1][0:2] == ('open_with', 'email_client'), msg=msg)
+				self.assertTrue(self.ui.mock_calls[-1][0] in ('_open_with_emailclient', '_open_with'), msg=msg)
+			elif type == 'smb' and os.name == 'nt':
+				self.assertTrue(self.ui.mock_calls[-1][0] == '_open_with_filebrowser', msg=msg)
 			else:
-				self.assertTrue(self.ui.mock_calls[-1][0:2] == ('open_with', 'web_browser'), msg=msg)
+				self.assertTrue(self.ui.mock_calls[-1][0] in ('_open_with_webbrowser', '_open_with'), msg=msg)
 			self.ui.mock_calls = [] # reset
 
 		# Some more tests that may not be covered above
@@ -655,14 +703,15 @@ class TestClickLink(tests.TestCase):
 			('file:///foo/bar', 'file'),
 			('mailto:foo@bar.com', 'mailto'),
 		):
+			#~ print ">> OPEN_URL %s (%s)" % (href, type)
 			self.ui.open_url(href)
 			msg = "open_url('%s')\nResulted in: %s" % (href, self.ui.mock_calls[-1])
 			if type == 'notebook':
 				self.assertTrue(self.ui.mock_calls[-1][0] == 'open_notebook', msg=msg)
 			elif type == 'file':
-				self.assertTrue(self.ui.mock_calls[-1][0] == 'open_file', msg=msg)
+				self.assertTrue(self.ui.mock_calls[-1][0] == '_open_with_webbrowser', msg=msg)
 			elif type == 'mailto':
-				self.assertTrue(self.ui.mock_calls[-1][0:2] == ('open_with', 'email_client'), msg=msg)
+				self.assertTrue(self.ui.mock_calls[-1][0] in ('_open_with_emailclient', '_open_with'), msg=msg)
 			self.ui.mock_calls = [] # reset
 
 		# TODO test plugin with custom handler
