@@ -1,18 +1,15 @@
-
 # -*- coding: utf-8 -*-
 
 # Copyright 2008 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 import tests
-from tests import TestCase
 
 import zim.plugins
 from zim.config import ConfigDict
 
 
-class TestPrintToBrowser(TestCase):
-
-	slowTest = True
+@tests.slowTest
+class TestPrintToBrowser(tests.TestCase):
 
 	def runTest(self):
 		'Test InlineCalculator plugin'
@@ -90,10 +87,10 @@ class StubUI(object):
 	ui_type = 'stub'
 
 	def __init__(self):
-		self.notebook = tests.get_test_notebook()
+		self.notebook = tests.new_notebook()
 		self.preferences = ConfigDict()
 		self.uistate = ConfigDict()
 
 	def connect(*a): pass
 
-	def connect_after(*a): pas
+	def connect_after(*a): pass
