@@ -11,7 +11,13 @@ import tests
 
 
 from zim import NotebookInterface
-from zim.config import XDG_CONFIG_HOME, ConfigDictFile, config_file
+from zim.config import XDG_CONFIG_HOME, ConfigFile, ConfigDictFile, config_file, get_config
+
+
+class FilterFailedToLoadPlugin(tests.LoggingFilter):
+
+	logger = 'zim'
+	message = 'Failed to load plugin'
 
 
 class TestProfiles(tests.TestCase):
@@ -21,11 +27,15 @@ class TestProfiles(tests.TestCase):
 		self.nb = tests.new_notebook(fakedir=path)
 		self.assertIsNone(self.nb.profile)
 
-		self.ui = NotebookInterface(self.nb)
-		self.ui.preferences.file.remove() # just in case
+		with FilterFailedToLoadPlugin():
+			self.ui = NotebookInterface(self.nb)
+
+		configfile = self.ui.preferences.file
+		configfile.file.remove() # just in case
 
 	def tearDown(self):
-		self.ui.preferences.file.remove()
+		configfile = self.ui.preferences.file
+		configfile.file.remove()
 
 	def profile_file(self, name):
 		return XDG_CONFIG_HOME.file('zim/profiles/%s.conf' % name.lower())
@@ -34,7 +44,8 @@ class TestProfiles(tests.TestCase):
 		file = self.profile_file(name)
 		file.remove()
 		self.assertFalse(file.exists())
-		ui = NotebookInterface() # use defaults set in this object
+		with FilterFailedToLoadPlugin():
+			ui = NotebookInterface() # use defaults set in this object
 		ui.preferences.change_file(file)
 		for section in preferences:
 			ui.preferences[section].update(preferences[section])
@@ -60,7 +71,8 @@ class TestProfiles(tests.TestCase):
 		self.nb.config['Notebook']['profile'] = 'TestProfile' # include some caps
 		self.assertEqual(self.nb.profile, 'TestProfile')
 
-		ui = NotebookInterface(self.nb)
+		with FilterFailedToLoadPlugin():
+			ui = NotebookInterface(self.nb)
 		self.assertEqual(ui.preferences.file.basename, 'testprofile.conf')
 		self.assertEqual(len(ui.plugins), 1)
 		self.assertEqual(
@@ -81,14 +93,15 @@ class TestProfiles(tests.TestCase):
 		# Save default
 		self.ui.preferences.write()
 		default = self.ui.preferences.file
-		self.assertTrue(default.exists())
+		self.assertTrue(default.file.exists())
 
 		# change the profile name, and reload the profile
 		# check that default got copied to new profile
 		self.nb.save_properties(profile='NewProfile')
 		self.assertEqual(self.nb.profile, 'NewProfile')
-		self.assertEqual(self.ui.preferences.file, file)
-		self.assertNotEqual(self.ui.preferences.file, default)
+		self.assertIsInstance(self.ui.preferences.file, ConfigFile)
+		self.assertEqual(self.ui.preferences.file.file, file)
+		self.assertNotEqual(self.ui.preferences.file.file, default)
 		self.ui.preferences.write() # ensure the preferences are saved
 
 		self.assertEqual(file.read(), default.read())
@@ -96,13 +109,14 @@ class TestProfiles(tests.TestCase):
 	def testReloadingPlugins(self):
 		'''Test correct plugins are kept when changing profile'''
 		# Ensure some plugins are loaded, including a independent one
+		n_default_plugins = len(self.ui.plugins)
 		self.ui.load_plugin('automount')
 		self.ui.preferences.write()
 
 		# create a profile just with some of the same plugins, but also
 		# a new one -- so we can test merging
 		names = self.ui.preferences['General']['plugins']
-		self.assertTrue(len(names) > 4)
+		self.assertEqual(len(names), n_default_plugins + 1)
 		self.assertNotIn('tableofcontents', names)
 		profile_plugins = [names[0], names[2], 'tableofcontents']
 		self.assertNotIn('automount', profile_plugins)
@@ -161,7 +175,7 @@ class TestProfiles(tests.TestCase):
 		calendar = self.ui.get_plugin('calendar')
 		calendar.preferences['test'] = 'old'
 		self.ui.preferences.write()
-		self.assertTrue(self.ui.preferences.file.exists())
+		self.assertTrue(self.ui.preferences.file.file.exists())
 
 		# Switch profile
 		self.nb.save_properties(profile='TestSyncing')
@@ -176,7 +190,7 @@ class TestProfiles(tests.TestCase):
 
 		# Ensure default config also has new config - but not all
 		# is overwritten
-		default = config_file('preferences.conf')
+		default = get_config('preferences.conf')
 		self.assertIn('automount', default['General']['plugins'])
 		self.assertEqual(default['AutomountPlugin']['test'], 'new')
 		self.assertEqual(default['CalendarPlugin']['test'], 'old')
@@ -193,11 +207,13 @@ class TestLoadingPlugins(tests.TestCase):
 	def setUp(self):
 		path = self.get_tmp_name()
 		self.nb = tests.new_notebook(fakedir=path)
-		self.ui = NotebookInterface(self.nb)
+		with FilterFailedToLoadPlugin():
+			self.ui = NotebookInterface(self.nb)
 		self.plugin_conf = self.ui.preferences['General']['plugins']
 
 	def tearDown(self):
-		self.ui.preferences.file.remove()
+		configfile = self.ui.preferences.file
+		configfile.file.remove()
 
 	def testLoadPlugin(self):
 		self.assertNotIn('automount', self.plugin_conf)
@@ -224,9 +240,10 @@ class TestLoadingPlugins(tests.TestCase):
 		names = self.plugin_conf[:]
 
 		self.ui.preferences.write()
-
-		myui = NotebookInterface()
-		# no notebook yet - only independent plugins are loaded
+		
+		with FilterFailedToLoadPlugin():
+			myui = NotebookInterface()
+			# no notebook yet - only independent plugins are loaded
 		self.assertEqual(len(myui.plugins), 1)
 		self.assertEqual(myui.plugins[0].plugin_key, 'automount')
 
