@@ -1545,6 +1545,21 @@ foo
 		result = cleanup(result)
 		self.assertEqual(result.tostring(), parsetree.tostring())
 
+		# copy partial
+		# line 33, offset 6 to 28 "try these **bold**, //italic//" in roundtrip page
+		wanted_tree = "<?xml version='1.0' encoding='utf-8'?>\n<zim-tree partial=\"True\">try these <strong>bold</strong>, <emphasis>italic</emphasis></zim-tree>"
+		wanted_text = "try these bold, italic" # no newline !
+		Clipboard.clear()
+		self.assertIsNone(Clipboard.get_parsetree())
+		start = buffer.get_iter_at_line_offset(33, 6)
+		end = buffer.get_iter_at_line_offset(33, 28)
+		buffer.select_range(start, end)
+		textview.emit('copy-clipboard')
+		result = Clipboard.get_parsetree(notebook, page)
+		self.assertIsNotNone(result)
+		self.assertEqual(result.tostring(), wanted_tree)
+		self.assertEqual(Clipboard.get_text(), wanted_text)
+
 		# cut
 		Clipboard.clear()
 		self.assertIsNone(Clipboard.get_parsetree())
@@ -1588,7 +1603,7 @@ foo
 		click(_('Copy _Link'))
 		self.assertEqual(Clipboard.get_text(), 'Bar')
 		tree = Clipboard.get_parsetree(pageview.ui.notebook, page)
-		self.assertEqual(tree.tostring(), '<?xml version=\'1.0\' encoding=\'utf-8\'?>\n<zim-tree><link href=":Bar">:Bar</link></zim-tree>')
+		self.assertEqual(tree.tostring(), '<?xml version=\'1.0\' encoding=\'utf-8\'?>\n<zim-tree><link href="Bar">Bar</link></zim-tree>')
 
 		page = tests.new_page_from_text('[[~//bar.txt]]')
 			# Extra '/' is in there to verify path gets parsed as File object
@@ -1696,7 +1711,7 @@ Baz
 		notebook = pageview.ui.notebook
 		self.assertIsNone(notebook.profile)
 		self.assertIsNone(pageview.style.profile)
-		self.assertEqual(pageview.style.file, default_file)
+		self.assertEqual(pageview.style.file.file, default_file)
 
 		# create a new style based on the default one, changing some properties
 		profile_file.remove()
@@ -1710,7 +1725,7 @@ Baz
 		notebook.save_properties(profile='testProfile')
 		self.assertEqual(notebook.profile, 'testProfile')
 		self.assertEqual(pageview.style.profile, 'testProfile')
-		self.assertEqual(pageview.style.file, profile_file)
+		self.assertEqual(pageview.style.file.file, profile_file)
 		self.assertEqual(pageview.style['TextView']['indent'], 50)
 		self.assertEqual(pageview.style['TextView']['font'], 'Sans 8')
 		self.assertEqual(pageview.style['TextView']['linespacing'], 10)
@@ -1729,7 +1744,7 @@ Baz
 
 		pageview = setUpPageView(notebook=notebook)
 		self.assertEqual(pageview.style.profile, 'testProfile')
-		self.assertEqual(pageview.style.file, profile_file)
+		self.assertEqual(pageview.style.file.file, profile_file)
 
 
 class TestPageviewDialogs(tests.TestCase):
@@ -1831,6 +1846,7 @@ dus bar bazzz baz
 		ui = MockUI()
 		ui.notebook.index = tests.MockObject()
 		ui.notebook.index.mock_method('list_pages', [])
+		ui.notebook.index.mock_method('walk', [])
 		pageview = tests.MockObject()
 		pageview.page = Path('Test:foo:bar')
 		textview = TextView({})
