@@ -43,6 +43,11 @@ class TestSearch(tests.TestCase):
 	def setUp(self):
 		self.notebook = tests.new_notebook()
 
+	def callback_check(self, selection, path):
+		self.assertIsInstance(selection, (SearchSelection, type(None)))
+		self.assertIsInstance(path, (Path, type(None)))
+		return True
+
 	def runTest(self):
 		'''Test search API'''
 		self.notebook.index.update()
@@ -54,7 +59,7 @@ class TestSearch(tests.TestCase):
 				QueryTerm('contentorname', 'foo'),
 				QueryTerm('contentorname', 'bar')
 			] )
-		results.search(query)
+		results.search(query, callback=self.callback_check)
 		#~ print results
 		self.assertTrue(len(results) > 0)
 		self.assertFalse(Path('TaskList:foo') in results)
@@ -69,7 +74,7 @@ class TestSearch(tests.TestCase):
 				QueryTerm('contentorname', 'TODO'),
 				QueryTerm('contentorname', 'bar', inverse=True)
 			] )
-		results.search(query)
+		results.search(query, callback=self.callback_check)
 		#~ print results
 		self.assertTrue(len(results) > 0)
 		self.assertTrue(Path('TaskList:foo') in results)
@@ -84,7 +89,7 @@ class TestSearch(tests.TestCase):
 				QueryTerm('contentorname', 'TODO'),
 				QueryTerm('contentorname', 'bar', inverse=True)
 			] )
-		results.search(query)
+		results.search(query, callback=self.callback_check)
 		#~ print results
 		self.assertTrue(len(results) > 0)
 		self.assertTrue(Path('TaskList:foo') in results)
@@ -100,7 +105,7 @@ class TestSearch(tests.TestCase):
 				QueryTerm('contentorname', 'TODO'),
 				QueryTerm('contentorname', 'bar')
 			] ] )
-		results.search(query)
+		results.search(query, callback=self.callback_check)
 		#~ print results
 		self.assertTrue(len(results) > 0)
 		self.assertTrue(Path('TaskList:foo') in results)
@@ -110,13 +115,13 @@ class TestSearch(tests.TestCase):
 		self.assertTrue(all(results.scores.values()))
 
 		query = Query('ThisWordDoesNotExistingInTheTestNotebook')
-		results.search(query)
+		results.search(query, callback=self.callback_check)
 		self.assertFalse(results)
 
 		query = Query('LinksTo: "Linking:Foo:Bar"')
 		self.assertTrue(query.root.operator == OPERATOR_AND)
 		self.assertEqual(query.root, [QueryTerm('linksto', 'Linking:Foo:Bar')])
-		results.search(query)
+		results.search(query, callback=self.callback_check)
 		#~ print results
 		self.assertTrue(Path('Linking:Dus:Ja') in results)
 		self.assertTrue(set(results.scores.keys()) == results)
@@ -125,14 +130,14 @@ class TestSearch(tests.TestCase):
 		query = Query('NOT LinksTo:"Linking:Foo:Bar"')
 		self.assertTrue(query.root.operator == OPERATOR_AND)
 		self.assertEqual(query.root, [QueryTerm('linksto', 'Linking:Foo:Bar', True)])
-		results.search(query)
+		results.search(query, callback=self.callback_check)
 		#~ print results
 		self.assertFalse(Path('Linking:Dus:Ja') in results)
 		self.assertTrue(set(results.scores.keys()) == results)
 		self.assertTrue(all(results.scores.values()))
 
 		query = Query('LinksTo:"NonExistingNamespace:*"')
-		results.search(query)
+		results.search(query, callback=self.callback_check)
 		self.assertFalse(results)
 
 		query = Query('LinksFrom: "Linking:Dus:Ja"')
@@ -141,25 +146,35 @@ class TestSearch(tests.TestCase):
 		query = Query('Links: "Linking:Dus:Ja"') # alias for LinksFrom
 		self.assertTrue(query.root.operator == OPERATOR_AND)
 		self.assertEqual(query.root, [QueryTerm('linksfrom', 'Linking:Dus:Ja')])
-		results.search(query)
+		results.search(query, callback=self.callback_check)
 		#~ print results
 		self.assertTrue(Path('Linking:Foo:Bar') in results)
 		self.assertTrue(set(results.scores.keys()) == results)
 		self.assertTrue(all(results.scores.values()))
 
 		query = Query('LinksFrom:"NonExistingNamespace:*"')
-		results.search(query)
+		results.search(query, callback=self.callback_check)
 		self.assertFalse(results)
 
 		query = Query('Namespace: "TaskList" fix')
 		self.assertTrue(query.root.operator == OPERATOR_AND)
 		self.assertEqual(query.root, [QueryTerm('namespace', 'TaskList'), QueryTerm('contentorname', 'fix')])
-		results.search(query)
+		results.search(query, callback=self.callback_check)
 		#~ print results
 		self.assertTrue(Path('TaskList:foo') in results)
 
+		for text in (
+			'Namespace: "Test:Foo Bar"',
+			'Namespace:"Test:Foo Bar"'
+		):
+			# check if space in page name works - found bug for 2nd form
+			query = Query(text)
+			results.search(query, callback=self.callback_check)
+			#~ print text, '>>' , results
+			self.assertTrue(Path('Test:Foo Bar:Dus Ja Hmm') in results)
+
 		query = Query('Namespace: "NonExistingNamespace"')
-		results.search(query)
+		results.search(query, callback=self.callback_check)
 		#~ print results
 		self.assertFalse(results)
 
@@ -169,13 +184,13 @@ class TestSearch(tests.TestCase):
 		query = Query('@tags')
 		self.assertTrue(query.root.operator == OPERATOR_AND)
 		self.assertEqual(query.root, [QueryTerm('tag', 'tags')])
-		results.search(query)
+		results.search(query, callback=self.callback_check)
 		#~ print results
 		self.assertTrue(Path('Test:tags') in results and len(results) == 2)
 			# Tasklist:all is the second match
 
 		query = Query('Tag: NonExistingTag')
-		results.search(query)
+		results.search(query, callback=self.callback_check)
 		self.assertFalse(results)
 
 		# TODO test ContentOrName versus Content
