@@ -9,6 +9,7 @@ import tests
 import os
 
 import zim
+from zim.errors import Error
 from zim.templates import *
 from zim.templates import GenericTemplate, \
 	TemplateParam, TemplateDict, TemplateFunction, PageProxy
@@ -24,9 +25,9 @@ class TestTemplateParam(tests.TestCase):
 		self.assertEquals(param.path, ['xxx', 'yyy'])
 		self.assertEquals(param.key, 'zzz')
 
-		self.assertRaises(TemplateSyntaxError, TemplateParam, '_settings.foo')
-		self.assertRaises(TemplateSyntaxError, TemplateParam, 'xxx._yyy.zzz')
-		self.assertRaises(TemplateSyntaxError, TemplateParam, 'xxx.y-y.zzz')
+		self.assertRaises(Error, TemplateParam, '_settings.foo')
+		self.assertRaises(Error, TemplateParam, 'xxx._yyy.zzz')
+		self.assertRaises(Error, TemplateParam, 'xxx.y-y.zzz')
 
 
 class TestTemplateDict(tests.TestCase):
@@ -123,16 +124,20 @@ class TestTemplateSet(tests.TestCase):
 			if format == 'templates':
 				continue # skip top level dir
 			files = [f for f in files if not f.startswith('.') and not '~' in f]
-			templates = list_templates(format)
+			files.sort()
 			self.assertTrue(len(files) > 0)
-			self.assertEqual(len(templates), len(files))
-			for file in templates.values():
-				#~ print files
+			templates = list_templates(format)
+			self.assertEqual([t[1] for t in templates], files)
+			for file in files:
 				file = os.path.join(dir, file)
-				tmpl = Template(file, format)
-				# Syntax errors will be raised during init
-				# TODO parameter check for these templates
-				#      ... run them with raise instead of param = None
+				input = open(file).readlines()
+				if format == 'plugins':
+					tmpl = GenericTemplate(input)
+				else:
+					tmpl = Template(input, format)
+					# Syntax errors will be raised during init
+					# TODO parameter check for these templates
+					#      ... run them with raise instead of param = None
 
 
 class TestPageProxy(tests.TestCase):
@@ -161,6 +166,7 @@ class TestTemplate(tests.TestCase):
 		input = u'''\
 Version [% zim.version %]
 <title>[% page.title %]</title>
+Created [% page.properties['Creation-Date'] %]
 <h1>[% notebook.name %]: [% page.name %]</h1>
 <h2>[% page.heading %]</h2>
 [% options.foo = "bar" %]
@@ -170,6 +176,7 @@ Option: [% options.foo %]
 		wantedresult = u'''\
 Version %s
 <title>Page Heading</title>
+Created TODAY
 <h1>Unnamed Notebook: FooBar</h1>
 <h2>Page Heading</h2>
 <p>
@@ -183,6 +190,7 @@ Option: bar
 ====== Page Heading ======
 **foo bar !**
 ''')
+		page.properties['Creation-Date'] = 'TODAY'
 		self.assertTrue(len(page.dump('html', linker=StubLinker())) > 0)
 		template = Template(input, 'html', linker=StubLinker())
 		result = template.process(notebook, page)
@@ -205,54 +213,54 @@ class TestTemplatePageIndexFuntion(tests.TestCase):
 		data = (
 ('Parent:Daughter', u"[% pageindex('Parent') %]", '''\
 <ul>
-<li><a href="page://:Parent:Child" title="Child">Child</a></li>
-<li><strong>Daughter</strong></li>
+<li><a href="page://:Parent:Child" title="Child" class="page">Child</a></li>
+<li><strong class="activepage">Daughter</strong></li>
 <ul>
-<li><a href="page://:Parent:Daughter:Granddaughter" title="Granddaughter">Granddaughter</a></li>
-<li><a href="page://:Parent:Daughter:Grandson" title="Grandson">Grandson</a></li>
-<li><a href="page://:Parent:Daughter:SomeOne" title="SomeOne">SomeOne</a></li>
+<li><a href="page://:Parent:Daughter:Granddaughter" title="Granddaughter" class="page">Granddaughter</a></li>
+<li><a href="page://:Parent:Daughter:Grandson" title="Grandson" class="page">Grandson</a></li>
+<li><a href="page://:Parent:Daughter:SomeOne" title="SomeOne" class="page">SomeOne</a></li>
 </ul>
-<li><a href="page://:Parent:Son" title="Son">Son</a></li>
+<li><a href="page://:Parent:Son" title="Son" class="page">Son</a></li>
 </ul>
 '''),
 
 ('Parent:Daughter:SomeOne', u"[% pageindex('Parent') %]", '''\
 <ul>
-<li><a href="page://:Parent:Child" title="Child">Child</a></li>
-<li><a href="page://:Parent:Daughter" title="Daughter">Daughter</a></li>
+<li><a href="page://:Parent:Child" title="Child" class="page">Child</a></li>
+<li><a href="page://:Parent:Daughter" title="Daughter" class="page">Daughter</a></li>
 <ul>
-<li><a href="page://:Parent:Daughter:Granddaughter" title="Granddaughter">Granddaughter</a></li>
-<li><a href="page://:Parent:Daughter:Grandson" title="Grandson">Grandson</a></li>
-<li><strong>SomeOne</strong></li>
+<li><a href="page://:Parent:Daughter:Granddaughter" title="Granddaughter" class="page">Granddaughter</a></li>
+<li><a href="page://:Parent:Daughter:Grandson" title="Grandson" class="page">Grandson</a></li>
+<li><strong class="activepage">SomeOne</strong></li>
 <ul>
-<li><a href="page://:Parent:Daughter:SomeOne:Bar" title="Bar">Bar</a></li>
-<li><a href="page://:Parent:Daughter:SomeOne:Foo" title="Foo">Foo</a></li>
+<li><a href="page://:Parent:Daughter:SomeOne:Bar" title="Bar" class="page">Bar</a></li>
+<li><a href="page://:Parent:Daughter:SomeOne:Foo" title="Foo" class="page">Foo</a></li>
 </ul>
 </ul>
-<li><a href="page://:Parent:Son" title="Son">Son</a></li>
+<li><a href="page://:Parent:Son" title="Son" class="page">Son</a></li>
 </ul>
 '''),
 
 ('Parent:Daughter:SomeOne', u"[% pageindex('Parent', FALSE, FALSE) %]", '''\
 <ul>
-<li><a href="page://:Parent:Child" title="Child">Child</a></li>
+<li><a href="page://:Parent:Child" title="Child" class="page">Child</a></li>
 <ul>
-<li><a href="page://:Parent:Child:Grandchild" title="Grandchild">Grandchild</a></li>
+<li><a href="page://:Parent:Child:Grandchild" title="Grandchild" class="page">Grandchild</a></li>
 </ul>
-<li><a href="page://:Parent:Daughter" title="Daughter">Daughter</a></li>
+<li><a href="page://:Parent:Daughter" title="Daughter" class="page">Daughter</a></li>
 <ul>
-<li><a href="page://:Parent:Daughter:Granddaughter" title="Granddaughter">Granddaughter</a></li>
-<li><a href="page://:Parent:Daughter:Grandson" title="Grandson">Grandson</a></li>
-<li><strong>SomeOne</strong></li>
+<li><a href="page://:Parent:Daughter:Granddaughter" title="Granddaughter" class="page">Granddaughter</a></li>
+<li><a href="page://:Parent:Daughter:Grandson" title="Grandson" class="page">Grandson</a></li>
+<li><strong class="activepage">SomeOne</strong></li>
 <ul>
-<li><a href="page://:Parent:Daughter:SomeOne:Bar" title="Bar">Bar</a></li>
-<li><a href="page://:Parent:Daughter:SomeOne:Foo" title="Foo">Foo</a></li>
+<li><a href="page://:Parent:Daughter:SomeOne:Bar" title="Bar" class="page">Bar</a></li>
+<li><a href="page://:Parent:Daughter:SomeOne:Foo" title="Foo" class="page">Foo</a></li>
 </ul>
 </ul>
-<li><a href="page://:Parent:Son" title="Son">Son</a></li>
+<li><a href="page://:Parent:Son" title="Son" class="page">Son</a></li>
 <ul>
-<li><a href="page://:Parent:Son:Granddaughter" title="Granddaughter">Granddaughter</a></li>
-<li><a href="page://:Parent:Son:Grandson" title="Grandson">Grandson</a></li>
+<li><a href="page://:Parent:Son:Granddaughter" title="Granddaughter" class="page">Granddaughter</a></li>
+<li><a href="page://:Parent:Son:Grandson" title="Grandson" class="page">Grandson</a></li>
 </ul>
 </ul>
 '''),
