@@ -39,6 +39,27 @@ WIKITEXT = File('tests/data/formats/wiki.txt').read() # Contains some unicode
 UTF8_COMMENT = u'Commit \u03b1\u03b2\u03b3'
 
 
+@tests.slowTest
+class TestVCS(tests.TestCase):
+
+	def testDetectVCS(self):
+		root = Dir(self.create_tmp_dir())
+		root.subdir('.bzr').touch()
+		self.assertEqual(VCS._detect_in_folder(root), ('bzr', root))
+
+		subdir = root.subdir('Foo/Bar')
+		subdir.touch()
+		self.assertEqual(VCS._detect_in_folder(subdir), ('bzr', root))
+
+		subroot = root.subdir('subroot')
+		subroot.subdir('.git').touch()
+		self.assertEqual(VCS._detect_in_folder(subroot), ('git', subroot))
+
+		subdir = subroot.subdir('Foo/Bar')
+		subdir.touch()
+		self.assertEqual(VCS._detect_in_folder(subdir), ('git', subroot))
+
+
 #####################################################
 #
 # BAZAAR BACKEND TEST
@@ -279,10 +300,16 @@ diff --git a/foo/bar/bar.txt b/foo/bar/bar.txt
 ''' )
 
 		vcs.revert()
-		self.assertEqual(vcs.get_status(), [
+		self.assertIn(vcs.get_status(), (
+[
 '# On branch master\n',
 'nothing to commit (working directory clean)\n'
-] )
+],
+[
+'# On branch master\n',
+'nothing to commit, working directory clean\n'
+],
+) )
 
 		file.write('second\nbaz\n')
 		vcs.commit_async('test 3')
@@ -348,7 +375,7 @@ test
 		self.assertIsInstance(diff, unicode)
 		vcs.commit(UTF8_COMMENT)
 		versions = vcs.list_versions()
-		self.assertTrue(UTF8_COMMENT in versions[-1][-1])
+		self.assertIn(UTF8_COMMENT, versions[-1][-1])
 		self.assertIsInstance(versions[-1][-1], unicode)
 
 
