@@ -94,7 +94,7 @@ For asynchronous actions see L{zim.async}.
 
 
 # Bunch of meta data, used at least in the about dialog
-__version__ = '0.59'
+__version__ = '0.60'
 __url__='http://www.zim-wiki.org'
 __author__ = 'Jaap Karssenberg <jaap.karssenberg@gmail.com>'
 __copyright__ = 'Copyright 2008 - 2013 Jaap Karssenberg <jaap.karssenberg@gmail.com>'
@@ -454,14 +454,16 @@ NOTE FOR BUG REPORTS:
 	zim trayicon (if any), and try again.
 ''')
 	elif cmd == 'server':
-		if 'standalone' in optsdict:
-			del optsdict['standalone']
-			# No daemon support for server, so no option doesn;t
+		standalone = optsdict.pop('standalone', False)
+			# No daemon support for server, so no option doesn't
 			# do anything for now
-
-		import zim.www
-		handler = zim.www.Server(*args, **optsdict)
-		handler.main()
+		gui = optsdict.pop('gui', False)
+		if gui:
+			import zim.gui.server
+			zim.gui.server.main(*args, **optsdict)
+		else:
+			import zim.www
+			zim.www.main(*args, **optsdict)
 	elif cmd == 'plugin':
 		import zim.plugins
 		try:
@@ -472,21 +474,41 @@ NOTE FOR BUG REPORTS:
 		module.main(*args)
 
 
-def ZimCmd():
+def ZimCmd(args=None):
 	'''Constructor to get a L{Application} object for zim itself
 	Use this object to spawn new instances of zim.
+	When C{args} is given the options "--standalone" and "-V" or "-D"
+	will be added automatically.
+	@param args: arguments to give to zim
 	@returns: a L{Application} object for zim itself
 	'''
-	# Note that Application takes care of using sys.executable when
-	# needed
 	from zim.applications import Application
 	if ZIM_EXECUTABLE.endswith('.exe'):
-		return Application((ZIM_EXECUTABLE,))
+		cmd = (ZIM_EXECUTABLE,)
 	elif sys.executable:
 		# If not an compiled executable, we assume it is python
-		# (Application class does this automatically for python scripts
+		# (Application class only does this automatically for scripts
 		# ending in .py)
-		return Application((sys.executable, ZIM_EXECUTABLE))
+		cmd = (sys.executable, ZIM_EXECUTABLE)
+
+	if not args:
+		return Application(cmd)
+
+	# TODO: if not standalone, call IPC directly rather than
+	#       first spawning a process
+	import zim.ipc
+	if not zim.ipc.in_child_process():
+		args = args + ('--standalone',)
+
+	# more detailed logging has lower number, so WARN > INFO > DEBUG
+	loglevel = logging.getLogger().getEffectiveLevel()
+	if loglevel <= logging.DEBUG:
+		args = args + ('-D',)
+	elif loglevel <= logging.INFO:
+		args = args + ('-V',)
+
+	return Application(cmd + args)
+
 
 
 class NotebookInterface(gobject.GObject):
@@ -526,7 +548,10 @@ class NotebookInterface(gobject.GObject):
 	__gsignals__ = {
 		'open-notebook': (gobject.SIGNAL_RUN_LAST, None, (object,)),
 		'preferences-changed': (gobject.SIGNAL_RUN_LAST, None, ()),
+		'initialize-notebook': (gobject.SIGNAL_RUN_LAST, None, (object,)),
 	}
+		# Consider making initialize-notebook a hook where handlers
+		# return a resolved notebook
 
 	ui_type = None
 
@@ -643,7 +668,7 @@ class NotebookInterface(gobject.GObject):
 			assert plugin in self.plugins
 			name = plugin.plugin_key
 
-		plugin.disconnect()
+		plugin.destroy()
 		self.plugins.remove(plugin)
 		logger.debug('Unloaded plugin %s', name)
 
@@ -731,13 +756,7 @@ class NotebookInterface(gobject.GObject):
 				nb, path = notebook, None
 
 			if not nb is None:
-				uri = nb.uri
-				for plugin in self.plugins:
-					try:
-						plugin.initialize_notebook(uri)
-					except Exception, error:
-						from zim.gui.widgets import ErrorDialog # HACK
-						ErrorDialog(None, error).run()
+				self.emit('initialize-notebook', nb.uri)
 				nb = get_notebook(nb)
 
 			if nb is None:
@@ -876,17 +895,6 @@ class NotebookInterface(gobject.GObject):
 			return True
 		index.update(callback=on_callback)
 
-	def spawn(self, *args):
-		'''Spawn a new instance of zim
-
-		Run a new zim process with commandline ares
-
-		@param args: any commandline args to pass to the new process
-
-		@todo: take this method outside this class
-		'''
-		zim = ZimCmd()
-		zim.spawn(args=args)
 
 # Need to register classes defining gobject signals
 gobject.type_register(NotebookInterface)
