@@ -94,20 +94,26 @@ class Application(object):
 	@staticmethod
 	def _lookup(cmd):
 		'''Lookup cmd in PATH'''
-		if os.name == 'nt' and not '.' in cmd:
-			cmd = cmd + '.exe'
-			# Automagically convert command names on windows
-
 		if zim.fs.isabs(cmd):
 			if zim.fs.isfile(cmd):
 				return cmd
 			else:
 				return None
+		elif os.name == 'nt':
+			# Check executable extensions from windows environment
+			extensions = get_environ_list('PATHEXT', '.com;.exe;.bat;.cmd')
+			for dir in get_environ_list('PATH'):
+				for ext in extensions:
+					file = os.sep.join((dir, cmd + ext))
+					if zim.fs.isfile(file) and os.access(file, os.X_OK):
+						return file
+			else:
+				return None
 		else:
-			# lookup in PATH
+			# On POSIX no extension is needed to make scripts executable
 			for dir in get_environ_list('PATH'):
 				file = os.sep.join((dir, cmd))
-				if zim.fs.isfile(file):
+				if zim.fs.isfile(file) and os.access(file, os.X_OK):
 					return file
 			else:
 				return None
@@ -195,7 +201,7 @@ class Application(object):
 		'''
 		cwd, argv = self._checkargs(cwd, args)
 		logger.info('Running: %s (cwd: %s)', argv, cwd)
-		p = subprocess.Popen(argv, cwd=cwd, stdout=subprocess.PIPE)
+		p = subprocess.Popen(argv, cwd=cwd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
 		stdout, stderr = p.communicate(input)
 		# TODO: handle ApplicationERror here as well ?
 
@@ -204,6 +210,7 @@ class Application(object):
 		#~ elif stderr:
 		if stderr:
 			logger.warn(stderr)
+			# TODO: allow user to get this error as well - e.g. for logging image generator cmd
 
 		return map(unicode, [line + '\n' for line in stdout.splitlines()])
 			# Explicit newline conversion, e.g. on windows \r\n -> \n
@@ -288,6 +295,8 @@ class WebBrowser(Application):
 			raise NotImplementedError, 'WebBrowser can not handle callback'
 
 		for url in args:
+			if isinstance(url, (zim.fs.File, zim.fs.Dir)):
+				url = url.uri
 			logger.info('Opening in webbrowser: %s', url)
 			self.controller.open(url)
 
