@@ -2,31 +2,51 @@
 
 # Copyright 2009 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
-'''This module contains the export functions for zim'''
+'''This module contains the export functions for zim.
+Main class is L{Exporter}, which implements the export action.
+See L{zim.gui.exportdialog} for the graphical user interface.
+'''
 
 import logging
 
-from zim.fs import *
+from zim.fs import Dir, File
 from zim.config import data_file
 from zim.formats import get_format, BaseLinker
 from zim.templates import get_template, Template
 from zim.notebook import Path, Page, IndexPage, PageNameError
 from zim.stores import encode_filename
-from zim.parsing import url_encode
+from zim.parsing import url_encode, url_decode
 
 logger = logging.getLogger('zim.exporter')
 
 
 class Exporter(object):
-	'''Class that handles an export action'''
+	'''Class that handles an export action.
+	The object instance holds all settings for the export. Once created
+	the actual export is done by calling L{export_all()} or by multiple
+	calls to L{export_page()}.
+	'''
 
 	def __init__(self, notebook, format, template=None,
 					index_page=None, document_root_url=None):
-		'''Constructor. The 'notebook' is the source for pages to be exported.
-		(The export target is given as an argument to export_all() or export().)
-		The 'format' and 'template' arguments determine the output format.
-		If 'index_page' is given a page index is generated and
-		'document_root_url' is used to prefix any file links that start with '/'.
+		'''Constructor.
+
+		Takes all input parameters on how to format the exported
+		content.
+
+		@param notebook: the L{Notebook} object
+		@param format: the output format as string, or formatting
+		module object
+		@param template: the template name as string, or a L{Template}
+		object, if C{None} no template is used
+		@param index_page: path name for the index page, if C{None} no
+		index page is generated
+		@param document_root_url: prefix for links that link to the
+		document root (e.g. URL for the document root on some server).
+		If C{None} document root files are considered the same as
+		other files.
+
+		@todo: check why index_page is a string and not a Path object
 		'''
 		self.notebook = notebook
 		self.document_root_url = document_root_url
@@ -52,10 +72,18 @@ class Exporter(object):
 			self.template.set_linker(self.linker)
 
 	def export_all(self, dir, callback=None):
-		'''Export all pages in the notebook to 'dir'. Attachments are copied
-		along. The function 'callback' will be called after each page with the
-		page object as single argument. If the callback returns False the
-		export will be cancelled.
+		'''Export all pages in the notebook
+
+		Will also copy all attachments and created a folder with icons
+		for checkboxes etc. So the resulting folder should contain all
+		the notebook information.
+
+		@param dir: a L{Dir} object for the target folder
+		@param callback: a callback function that will be called after
+		each page being exported with the page object as single argument.
+		If the function returns C{False} the export is canceled.
+
+		@returns: C{False} if the action was cancelled, C{True} otherwise
 		'''
 		logger.info('Exporting notebook to %s', dir)
 		self.linker.target_dir = dir # Needed to resolve icons
@@ -63,9 +91,15 @@ class Exporter(object):
 		# Copy icons
 		for name in ('checked-box', 'unchecked-box', 'xchecked-box'):
 			icon = data_file('pixmaps/%s.png' % name)
-			file = dir.file('_icons/'+name+'.png')
+			file = dir.file('_resources/' + name + '.png')
 			icon.copyto(file)
 
+		# Copy template resources (can overwrite icons)
+		if self.template and self.template.resources_dir \
+		and self.template.resources_dir.exists():
+			resources = dir.subdir('_resources')
+			self.template.resources_dir.copyto(resources)
+
 		# Set special pages
 		if self.index_page:
 			indexpage = Page(Path(self.index_page))
@@ -87,7 +121,7 @@ class Exporter(object):
 					pages['next'] = next
 					self.export_page(dir, current, pages, use_namespace=True)
 					if callback and not callback(current):
-						logger.warn('Export cancelled')
+						logger.warn('Export canceled')
 						return False
 
 		prev, current, next = current, next, None # shift once more
@@ -96,7 +130,7 @@ class Exporter(object):
 			pages['next'] = next
 			self.export_page(dir, current, pages, use_namespace=True)
 			if callback and not callback(current):
-				logger.warn('Export cancelled')
+				logger.warn('Export canceled')
 				return False
 
 		# Generate index page
@@ -113,19 +147,29 @@ class Exporter(object):
 		return True
 
 	def export_page(self, dir, page, pages=None, use_namespace=False, filename=None, dirname=None):
-		'''Export 'page' to a file below 'dir'. Attachments wil also be
-		copied along.
+		'''Export a single page and copy it's attachments. Does not
+		include sub-pages.
 
-		If only a page is given that output file will have the same
-		basename as the page. If 'use_namespace' is set to True the
-		path below 'dir' will be determined by the namespace of 'page'.
-		The attachment directory will match the name of the file but
-		without extension.
+		@param dir: a L{Dir} object for the target folder
+		@param page: the L{page} to export
+		@param pages: dict with special pages that is passed on to the
+		template
+		@param use_namespace: when C{False} the export file will be
+		placed directly in C{dir}, when C{True} the page will be put
+		in a sub-folder structure that reflects the namespace of the
+		page
+		@param filename: alternative filename to use for the export
+		file instead of the page name. If needed the appropriate
+		file extension is added to the name.
+		@param dirname: alternative name to use for the folder with
+		attachments, if C{None} it takes the filename without the
+		extension.
 
-		Alternatively when a filename is given it will be used. If
-		needed the apropriate file extension is added to the name.
-		Similar the dirname option can be used to specify the directory
-		for attachments, otherwise it is derived from the filename.
+		@todo: Get rid of C{use_namespace} by creating a seperate
+		function for that. Maybe create a variant that does a single
+		page in an folder directly and a higher level function that
+		does an entire namespace including sub-pages
+		@todo: change filename and dirname in File and Dir arguments
 		'''
 		logger.info('Exporting %s', page.name)
 
@@ -144,7 +188,7 @@ class Exporter(object):
 
 		file = dir.file(filename)
 		attachments = self.notebook.get_attachments_dir(page)
-		self.linker.set_base(attachments.dir)
+		self.linker.set_base(attachments.dir) # parent of attachment dir
 			# FIXME, assuming standard file store layout to get correct relative links
 		self.linker.target_file = file
 
@@ -163,22 +207,29 @@ class Exporter(object):
 			# - also include "attachments" in the root namespace
 
 	def export_page_to_fh(self, fh, page, pages=None):
-		'''Export 'page' and print the output to open file handle 'fh'.
-		(Does not do anything with attachments.)
+		'''Output the export content for a single page to an open file
+
+		@param fh: an open file object, or any object that has a
+		C{writelines()} method
+		@param page: the L{page} to export
+		@param pages: dict with special pages that is passed on to the
+		template
 		'''
 		if self.template is None:
 			self.linker.set_path(page)
 			lines = page.dump(self.format, linker=self.linker)
 		else:
 			lines = self.template.process(self.notebook, page, pages)
-		fh.writelines(l.encode('utf-8') for l in lines)
+		fh.writelines(lines)
 
 
 class StaticLinker(BaseLinker):
-	'''Linker object for exporting a single page. It links files, images
-	and icons with absolute or relative file paths (based on whether the
-	format supports relative links or not). Other pages are linked as
-	files.
+	'''Linker object to be used by the template when exporting.
+	It links files, images and icons with absolute or relative file paths
+	(based on whether the format supports relative links or not).
+	Other pages are linked as files.
+
+	See L{BaseLinker} for the API docs.
 	'''
 
 	def __init__(self, format, notebook, path=None, document_root_url=None):
@@ -192,10 +243,17 @@ class StaticLinker(BaseLinker):
 		self.target_file = None
 		self._extension = '.' + format.info['extension']
 
+	def resource(self, path):
+		if self.target_dir and self.target_file:
+			file = self.target_dir.file('_resources/'+path)
+			return self._filepath(file, self.target_file.dir)
+		else:
+			path
+
 	def icon(self, name):
 		if self.target_dir and self.target_file:
-			file = self.target_dir.file('_icons/'+name+'.png')
-			return self._filepath(file, self.target_file)
+			file = self.target_dir.file('_resources/'+name+'.png')
+			return self._filepath(file, self.target_file.dir)
 		else:
 			return BaseLinker.icon(self, name)
 
@@ -227,6 +285,15 @@ class StaticLinker(BaseLinker):
 			#~ print '>>>', path
 			return url_encode(path.replace(' ', '_'))
 
+	def resolve_file(self, link):
+		try:
+			file = self.notebook.resolve_file(link, self.path)
+		except:
+			# typical error is a non-local file:// uri
+			return None
+		else:
+			return file
+
 	def link_file(self, link):
 		if self.document_root_url and link.startswith('/'):
 			return ''.join((self.document_root_url.rstrip('/'), link))
@@ -255,6 +322,7 @@ class StaticLinker(BaseLinker):
 			uri, path = url.split('?')
 			# FIXME: code below is not robust because we don't know the
 			# storage mode of linked notebook...
+			path = url_decode(path) # was already encoded by interwiki_link()
 			path = encode_filename(path).replace(' ', '_')
 			return uri + '/' + url_encode(path) + '.txt'
 		else:
