@@ -140,7 +140,10 @@ def parsetree_from_selectiondata(selectiondata, notebook=None, path=None):
 		# FIXME some apps drop text/uri-list as a text/plain mimetype
 		# try to catch this situation by a check here
 		text = selectiondata.get_text()
-		return get_format('plain').Parser().parse(text.decode('utf-8'))
+		if text:
+			return get_format('plain').Parser().parse(text.decode('utf-8'))
+		else:
+			return None
 	elif targetname in IMAGE_TARGET_NAMES:
 		# save image
 		pixbuf = selectiondata.get_pixbuf()
@@ -153,8 +156,11 @@ def parsetree_from_selectiondata(selectiondata, notebook=None, path=None):
 			dir.touch()
 
 		format, extension = _get_image_info(targetname)
-		if format is None:
-			format, extension = 'png', 'png' # default to png format
+		if format is None or format == 'bmp':
+			# default to png format
+			# special casing bmp since many window apps use it internally
+			# but is quite large to store, so compress by using png
+			format, extension = 'png', 'png'
 
 		file = dir.new_file('pasted_image.%s' % extension)
 		logger.debug("Saving image from clipboard to %s", file)
@@ -302,6 +308,7 @@ class ParseTreeItem(ClipboardItem):
 		@param clipboard: a C{gtk.Clipboard} objet
 		@param selectiondata: a C{gtk.SelectionData} object to set the data on
 		@param id: target id for the requested data format
+		@param a: any additional arguments are discarded
 		'''
 		logger.debug("Clipboard requests data as '%s', we have a parsetree", selectiondata.target)
 		if id == PARSETREE_TARGET_ID:
@@ -335,6 +342,8 @@ class ParseTreeItem(ClipboardItem):
 				text = ''.join( dumper.dump(self.parsetree) ).encode('utf-8')
 				parser = get_format('wiki').Parser()
 				parsetree = parser.parse(text)
+				if self.parsetree.ispartial:
+					parsetree.getroot().attrib['partial'] = True
 				#--
 			else:
 				parsetree = self.parsetree
@@ -370,6 +379,7 @@ class PageLinkItem(ClipboardItem):
 		@param clipboard: a C{gtk.Clipboard} objet
 		@param selectiondata: a C{gtk.SelectionData} object to set the data on
 		@param id: target id for the requested data format
+		@param a: any additional arguments are discarded
 		'''
 		logger.debug("Clipboard requests data as '%s', we have a pagelink", selectiondata.target)
 		if id == INTERNAL_PAGELIST_TARGET_ID:
