@@ -33,6 +33,9 @@ class ExportDialog(Assistant):
 			if self.uistate[k] and not self.uistate[k].isspace():
 				options[k] = self.uistate[k]
 
+		options['format'] = \
+			zim.formats.canonical_name(options['format'])
+
 		if options['template'] == '__file__':
 			options['template'] = self.uistate['template_file']
 
@@ -45,6 +48,13 @@ class ExportDialog(Assistant):
 			ErrorDialog(self, error).run()
 			return False
 
+		index = self.ui.notebook.index
+		if index.updating:
+			with ProgressBarDialog(self, _('Updating index')) as dialog: # T: Title of progressbar dialog
+				index.ensure_update(callback=lambda p: dialog.pulse(p.name))
+				if dialog.cancelled:
+					return False
+
 		if self.uistate['selection'] == 'all':
 			dir = Dir(self.uistate['output_folder'])
 			if dir.exists() and len(dir.list()) > 0:
@@ -58,12 +68,9 @@ class ExportDialog(Assistant):
 				if not ok:
 					return False
 
-			dialog = ProgressBarDialog(self, _('Exporting notebook'))
-				# T: Title for progressbar window
-				# TODO make progressbar a context manager - now it stays alive in case of an error during the export
-			dialog.show_all()
-			exporter.export_all(dir, callback=lambda p: dialog.pulse(p.name))
-			dialog.destroy()
+			with ProgressBarDialog(self, _('Exporting notebook')) as dialog: # T: Title for progressbar window
+				exporter.export_all(dir, callback=lambda p: dialog.pulse(p.name))
+
 		elif self.uistate['selection'] == 'selection':
 			pass # TODO
 		elif self.uistate['selection'] == 'page':
@@ -166,11 +173,11 @@ class FormatPage(AssistantPage):
 		# Set template list based on selected format
 		def set_templates(self):
 			format = self.form['format']
+			format = zim.formats.canonical_name(format)
 			combobox = self.form.widgets['template']
 			combobox.get_model().clear()
 
-			templates = zim.templates.list_templates(format)
-			for name in sorted(templates):
+			for name, _ in zim.templates.list_templates(format):
 				combobox.append_text(name)
 			combobox.append_text(self.CHOICE_OTHER)
 			combobox.set_sensitive(True)
@@ -254,9 +261,9 @@ class OutputPage(AssistantPage):
 		# Switch between folder selection or file selection based
 		# on whether we selected full notebook or single page in the
 		# first page
-		self.uistate.setdefault('output_folder', '')
+		self.uistate.setdefault('output_folder', '', Dir)
 		self.uistate.setdefault('index_page', '')
-		self.uistate.setdefault('output_file', '')
+		self.uistate.setdefault('output_file', '', File)
 
 		show_file = self.uistate.get('selection') == 'page'
 		if show_file:
@@ -273,9 +280,14 @@ class OutputPage(AssistantPage):
 		if show_file:
 			basename = self.uistate['selected_page'].basename
 			ext = zim.formats.get_format(self.uistate['format']).info['extension']
-			file = File('~/' + encode_filename(basename  + '.' + ext))
+
+			if self.uistate['output_file'] \
+			and isinstance(self.uistate['output_file'], File):
+				dir = self.uistate['output_file'].dir
+				file = dir.file(encode_filename(basename  + '.' + ext))
+			else:
+				file = File('~/' + encode_filename(basename  + '.' + ext))
 			self.uistate['output_file'] = file
-			# TODO remember last file output folder
 
 		self.form['file'] = self.uistate['output_file']
 		self.form['folder'] = self.uistate['output_folder']
