@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2008 Jaap Karssenberg <jaap.karssenberg@gmail.com>
+# Copyright 2008-2013 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 '''
 This module contains the main Notebook class and related classes.
@@ -81,6 +81,9 @@ import logging
 
 import gobject
 
+
+from .base import Object
+
 import zim.fs
 from zim.fs import File, Dir
 from zim.errors import Error, TrashNotSupportedError
@@ -649,7 +652,7 @@ class PageReadOnlyError(Error):
 _first_char_re = re.compile(r'^\W', re.UNICODE)
 
 
-class Notebook(gobject.GObject):
+class Notebook(Object):
 	'''Main class to access a notebook
 
 	This class defines an API that proxies between backend L{zim.stores}
@@ -667,6 +670,8 @@ class Notebook(gobject.GObject):
 	@signal: C{profile-changed ()}: emitted when the profile is changed,
 	means that the preferences need to be loaded again as well
 	@signal: C{properties-changed ()}: emitted when properties changed
+	@signal: C{suggest-link (path, text)}: hook that is called when trying
+	to resolve links
 
 	@note: For store_async() the 'page-stored' signal is emitted
 	after scheduling the store, but potentially before it was really
@@ -712,7 +717,9 @@ class Notebook(gobject.GObject):
 		'deleted-page': (gobject.SIGNAL_RUN_LAST, None, (object,)),
 		'profile-changed': (gobject.SIGNAL_RUN_FIRST, None, ()),
 		'properties-changed': (gobject.SIGNAL_RUN_FIRST, None, ()),
+		'suggest-link': (gobject.SIGNAL_RUN_LAST, None, (object, object)),
 	}
+	__hooks__ = ('suggest-link',)
 
 	properties = (
 		('name', 'string', _('Name')), # T: label for properties dialog
@@ -727,7 +734,6 @@ class Notebook(gobject.GObject):
 
 	def __init__(self, dir=None, file=None, config=None, index=None):
 		assert not (dir and file), 'BUG: can not provide both dir and file '
-		gobject.GObject.__init__(self)
 		self._namespaces = []	# list used to resolve stores
 		self._stores = {}		# dict mapping namespaces to stores
 		self.namespace_properties = HierarchicDict({
@@ -751,7 +757,19 @@ class Notebook(gobject.GObject):
 		if dir:
 			assert isinstance(dir, Dir)
 			self.dir = dir
-			self.readonly = not dir.iswritable()
+			#~ self.readonly = not dir.iswritable()
+
+			# Test access - (iswritable turns out to be unreliable
+			# for folders on windows..)
+			f = dir.file('.zim/tmp')
+			try:
+				f.write('Test')
+				f.remove()
+			except:
+				logger.info('Notebook readonly')
+				self.readonly = True
+			else:
+				self.readonly = False
 
 			if self.config is None:
 				self.config = ConfigDictFile(dir.file('notebook.zim'))
@@ -1107,40 +1125,13 @@ class Notebook(gobject.GObject):
 			else:
 				return parent.basename + ':' + href.relname(parent)
 
-	def register_hook(self, name, handler):
-		'''Register a handler method for a specific hook
-
-		@todo: move this method to a more generic signal module
-		'''
-		register = '_register_%s' % name
-		if not hasattr(self, register):
-			setattr(self, register, [])
-		getattr(self, register).append(handler)
-
-	def unregister_hook(self, name, handler):
-		'''Remove a handler method for a specific hook
-
-		@todo: move this method to a more generic signal module
-		'''
-		register = '_register_%s' % name
-		if hasattr(self, register):
-			getattr(self, register).remove(handler)
-
 	def suggest_link(self, source, word):
 		'''Suggest a link Path for 'word' or return None if no suggestion is
 		found. By default we do not do any suggestion but plugins can
-		register handlers to add suggestions. See 'register_hook()' to
-		register a handler. The hook name is "suggest_link".
+		register handlers to add suggestions using the 'C{suggest-link}'
+		signal.
 		'''
-		if not  hasattr(self, '_register_suggest_link'):
-			return None
-
-		for handler in self._register_suggest_link:
-			link = handler(source, word)
-			if not link is None:
-				return link
-		else:
-			return None
+		return self.emit('suggest-link', source, word)
 
 	@staticmethod
 	def cleanup_pathname(name, purge=False):
@@ -1391,9 +1382,22 @@ class Notebook(gobject.GObject):
 		# This assumption may change in the future.
 		assert page.valid, 'BUG: page object no longer valid'
 		self.emit('store-page', page)
+
+		# FIXME can signal framework help to make this code bit more compact ?
+		def mycallback(ok, error, exc_info, data):
+			if callback:
+				try:
+					if data:
+						callback(ok, error, exc_info, data)
+					else:
+						callback(ok, error, exc_info)
+				except:
+					logger.exception('Exception in callback after store-async:')
+			if ok:
+				self.emit('stored-page', page)
+
 		store = self.get_store(page)
-		store.store_page_async(page, self.lock, callback, data)
-		self.emit('stored-page', page)
+		store.store_page_async(page, self.lock, mycallback, data)
 
 	def revert_page(self, page):
 		'''Reload the page from the storage backend, discarding all
@@ -2066,9 +2070,6 @@ class Notebook(gobject.GObject):
 		self.config.write()
 		logger.info('Notebook update done')
 
-# Need to register classes defining gobject signals
-gobject.type_register(Notebook)
-
 
 class Path(object):
 	'''Class representing a page name in the notebook
