@@ -1,11 +1,13 @@
 # -*- coding: utf-8 -*-
 #
 # Copyright 2010 Thorsten Hackbarth <thorsten.hackbarth@gmx.de>
-#           2011 Jaap Karssenberg <jaap.karssenberg@gmail.com>
+#           2011-2013 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 # License:  same as zim (gpl)
 #
 # ChangeLog
-# 2012-08-17 Added mimetype icons, statusbar toggle button, and gio monitor support
+# 2013-03-03 Change to new plugin extension structure (Jaap)
+# 2013-02-25 Added zooming icon size, made icon rendering more robust (Jaap)
+# 2012-08-17 Added mimetype icons, statusbar toggle button, and gio monitor support (Jaap)
 # 2012-07-20 Updated code for pane uistate (Jaap)
 # 2012-04-17 Allow drag&drop when folder does not exist yet + fix drag&drop on windows (Jaap)
 # 2012-02-29 Further work on making iconview look nice and support drag&drop (Jaap)
@@ -21,17 +23,16 @@
 # 2010-06-29 1st working version
 #
 # TODO:
+# [ ] Action for deleting files in context menu
+# [ ] Copy / cut files in context menu
+# [ ] Button to clean up the folder - only show when the folder is empty
 # [x] GIO watcher to detect folder update - add API to zim.fs for watcher ?
-# [ ] Allow more than 1 thread for thumbnailing
+# [ ] Allow more than 1 thread for thumbnailing ?
 # [ ] Can we cache image to thumb mapping (or image MD5) to spead up ?
 # [ ] Dont thumb small images
 # [x] Mimetype specific icons
 # [ ] Restore ImageMagick thumbnailer
 # [ ] Use thumbnailers/settings from gnome or other DEs ?
-# [ ] Action for deleting files in context menu
-# [ ] Copy / cut files in context menu
-# [ ] Button to clean up the folder - only show when the folder is empty
-
 
 '''Zim plugin to display files in attachments folder.'''
 
@@ -54,7 +55,8 @@ except ImportError:
 import zim
 import zim.config # Asserts HOME is defined
 
-from zim.plugins import PluginClass
+from zim.plugins import PluginClass, WindowExtension, extends
+from zim.actions import toggle_action
 
 from zim.fs import File, Dir, format_file_size
 from zim.errors import Error
@@ -74,31 +76,6 @@ from zim.gui.clipboard import \
 logger = logging.getLogger('zim.plugins.attachmentbrowser')
 
 
-ui_toggle_actions = (
-	# name, stock id, label, accelerator, tooltip, readonly
-	('toggle_fileview', gtk.STOCK_DIRECTORY, _('AttachmentBrowser'),  '', 'Show Attachment Folder',False, True), # T: menu item
-)
-
-
-#Menubar and toolbar
-ui_xml = '''
-<ui>
-	<menubar name='menubar'>
-		<menu action='view_menu'>
-			<placeholder name="plugin_items">
-				<menuitem action="toggle_fileview" />
-			</placeholder>
-		</menu>
-	</menubar>
-	<toolbar name='toolbar'>
-		<placeholder name='tools'>
-			<toolitem action='toggle_fileview'/>
-		</placeholder>
-	</toolbar>
-</ui>
-'''
-
-
 # freedesktop.org spec
 LOCAL_THUMB_STORAGE = Dir('~/.thumbnails')
 LOCAL_THUMB_STORAGE_NORMAL = LOCAL_THUMB_STORAGE.subdir('normal')
@@ -108,9 +85,13 @@ LOCAL_THUMB_STORAGE_FAIL = LOCAL_THUMB_STORAGE.subdir('fail/zim-%s' % zim.__vers
 THUMB_SIZE_NORMAL = 128
 THUMB_SIZE_LARGE = 256
 
-# For plugin -- TODO make configable / zoomable
-ICON_SIZE = 64
-PREVIEW_SIZE = 128
+# For plugin
+MIN_ICON_SIZE = 16
+DEFAULT_ICON_SIZE = 64
+
+
+_last_warning_missing_icon = None
+	# used to surpress redundant logging
 
 
 def get_mime_icon(file, size):
@@ -126,6 +107,8 @@ def get_mime_icon(file, size):
 		logger.exception('Failed to query info for file: %s', file)
 		return None
 
+	global _last_warning_missing_icon
+
 	if isinstance(icon, gio.ThemedIcon):
 		names = icon.get_names()
 		icon_theme = gtk.icon_theme_get_default()
@@ -134,7 +117,9 @@ def get_mime_icon(file, size):
 			if icon_info:
 				return icon_info.load_icon()
 			else:
-				logger.debug('Missing icons in icon theme: %s', names)
+				if _last_warning_missing_icon != names:
+					logger.debug('Missing icons in icon theme: %s', names)
+					_last_warning_missing_icon = names
 				return None
 		except gobject.GError:
 			logger.exception('Could not load icon for file: %s', file)
@@ -143,6 +128,78 @@ def get_mime_icon(file, size):
 		return None
 
 
+def get_mime_description(mimetype):
+	# TODO move this to zim.fs or something
+	#
+	# Check XML file /usr/share/mime/MEDIA/SUBTYPE.xml
+	# Find element "comment" with "xml:lang" attribute for the locale
+	# Etree fills in the namespaces which obfuscates the names
+	from zim.config import XDG_DATA_DIRS
+
+	media, subtype = mimetype.split('/', 1)
+	for dir in XDG_DATA_DIRS:
+		file = dir.file(('mime', media, subtype + '.xml'))
+		if file.exists():
+			return _read_comment_from(file)
+	else:
+		return None
+
+
+def _read_comment_from(file):
+	import locale
+	from zim.formats import ElementTreeModule as et
+
+	mylang, enc = locale.getdefaultlocale()
+	xmlns = "{http://www.w3.org/XML/1998/namespace}"
+	xml = et.parse(file.path)
+	fallback = []
+	#~ print "FIND COMMENT", file, mylang
+	for elt in xml.getroot():
+		if elt.tag.endswith('comment'):
+			lang = elt.attrib.get(xmlns+'lang', '')
+			if lang == mylang:
+				return elt.text
+			elif not lang or mylang.startswith(lang+'_'):
+				fallback.append((lang, elt.text))
+			else:
+				pass
+	else:
+		#~ print "FALLBACK", fallback
+		if fallback:
+			fallback.sort()
+			return fallback[-1][1] # longest match
+		else:
+			return None
+
+
+def render_file_icon(widget, size):
+	# Sizes defined in gtk source,
+	# gtkiconfactory.c for gtk+ 2.18.9
+	#
+	#	(gtk.ICON_SIZE_MENU, 16),
+	#	(gtk.ICON_SIZE_BUTTON, 20),
+	#	(gtk.ICON_SIZE_SMALL_TOOLBAR, 18),
+	#	(gtk.ICON_SIZE_LARGE_TOOLBAR, 24),
+	#	(gtk.ICON_SIZE_DND, 32),
+	#	(gtk.ICON_SIZE_DIALOG, 48),
+	#
+	# We expect sizes in list: 16, 32, 64, 128
+	# But only give back 16 or 32, bigger icons
+	# do not look good
+	assert size in (16, 32, 64, 128)
+	if size == 16:
+		pixbuf = widget.render_icon(gtk.STOCK_FILE, gtk.ICON_SIZE_MENU)
+	else:
+		pixbuf = widget.render_icon(gtk.STOCK_FILE, gtk.ICON_SIZE_DND)
+
+	# Not sure how much sizes depend on theming,
+	# so we scale down if needed, do not scale up
+	if pixbuf.get_width() > size or pixbuf.get_height() > size:
+		return pixbuf.scale_simple(size, size, gtk.gdk.INTERP_BILINEAR)
+	else:
+		return pixbuf
+
+
 def is_hidden_file(file):
 	# FIXME put this in zim.fs
 	if not os.name == 'nt':
@@ -166,8 +223,6 @@ def is_hidden_file(file):
 
 class AttachmentBrowserPlugin(PluginClass):
 
-	TAB_NAME = _('Attachments') # T: label for attachment browser pane
-
 	plugin_info = {
 		'name': _('Attachment Browser'), # T: plugin name
 		'description': _('''\
@@ -185,7 +240,6 @@ This plugin is still under development.
 		('pane', 'choice', _('Position in the window'), BOTTOM_PANE, PANE_POSITIONS),
 			# T: option for plugin preferences
 
-	#	('icon_size', 'int', _('Icon size [px]'), [ICON_SIZE_MIN,128,ICON_SIZE_MAX]), # T: preferences option
 	#	('preview_size', 'int', _('Tooltip preview size [px]'), (THUMB_SIZE_MIN,480,THUMB_SIZE_MAX)), # T: input label
 	#	('thumb_quality', 'int', _('Preview jpeg Quality [0..100]'), (0,50,100)), # T: input label
 	#~	('use_imagemagick', 'bool', _('Use ImageMagick for thumbnailing'), False), # T: input label
@@ -195,70 +249,100 @@ This plugin is still under development.
 	#~ def check_dependencies(klass):
 		#~ return [("ImageMagick",Application(('convert',None)).tryexec())]
 
-	def initialize_ui(self, ui):
+
+@extends('MainWindow')
+class MainWindowExtension(WindowExtension):
+
+	TAB_NAME = _('Attachments') # T: label for attachment browser pane
+
+	uimanager_xml = '''
+	<ui>
+		<menubar name='menubar'>
+			<menu action='view_menu'>
+				<placeholder name="plugin_items">
+					<menuitem action="toggle_fileview" />
+				</placeholder>
+			</menu>
+		</menubar>
+		<toolbar name='toolbar'>
+			<placeholder name='tools'>
+				<toolitem action='toggle_fileview'/>
+			</placeholder>
+		</toolbar>
+	</ui>
+	'''
+
+	def __init__(self, plugin, window):
+		WindowExtension.__init__(self, plugin, window)
+		self.preferences = plugin.preferences
+		self.uistate = plugin.uistate
 		self._monitor = None
-		self._block_toggle = False
-		if self.ui.ui_type == 'gtk':
-			self.ui.add_toggle_actions(ui_toggle_actions, self)
-			#self.ui.add_actions(ui_actions, self)
-			self.ui.add_ui(ui_xml, self)
 
-	def finalize_ui(self, ui):
-		if self.ui.ui_type == 'gtk':
-			self.widget = AttachmentBrowserPluginWidget(self, self.preferences)
+		# Init statusbar button
+		self.statusbar_frame = gtk.Frame()
+		self.statusbar_frame.set_shadow_type(gtk.SHADOW_IN)
+		self.window.statusbar.pack_end(self.statusbar_frame, False)
 
-			self.statusbar_frame = gtk.Frame()
-			self.statusbar_frame.set_shadow_type(gtk.SHADOW_IN)
-			self.ui.mainwindow.statusbar.pack_end(self.statusbar_frame, False)
+		self.statusbar_button = gtk.ToggleButton('<attachments>') # translated below
+		button_set_statusbar_style(self.statusbar_button)
 
-			self.statusbar_button = gtk.ToggleButton('<attachments>') # translated below
-			button_set_statusbar_style(self.statusbar_button)
+		self.statusbar_button.set_use_underline(True)
+		self.__class__.toggle_fileview.connect_actionable(
+			self, self.statusbar_button)
 
-			self.statusbar_button.set_use_underline(True)
-			self.statusbar_button.connect_after('toggled',
-				lambda o: self.toggle_fileview(enable=o.get_active()) )
-			self.statusbar_frame.add(self.statusbar_button)
-			self.statusbar_frame.show_all()
+		self.statusbar_frame.add(self.statusbar_button)
+		self.statusbar_frame.show_all()
 
-			self.do_preferences_changed()
+		# Init browser widget
+		opener = self.window.get_resource_opener()
+		self.widget = AttachmentBrowserPluginWidget(self, opener, self.preferences)
+			# FIXME FIXME FIXME - get rid of ui object here
+		self.connectto(plugin, 'preferences-changed')
+		self.on_preferences_changed()
 
-			if self.ui.page:
-				self.on_open_page(self.ui, self.ui.page, self.ui.page)
-			self.connectto(self.ui, 'open-page')
+		# XXX
+		if self.window.ui.page:
+			self.on_open_page(self.window.ui, self.window.ui.page, self.window.ui.page)
+		self.connectto(self.window.ui, 'open-page')
 
-			self.connectto(self.ui.mainwindow, 'pane-state-changed')
+		self.connectto(self.window, 'pane-state-changed')
 
+	def on_preferences_changed(self):
+		try:
+			self.window.remove(self.widget)
+		except ValueError:
+			pass
+		self.window.add_tab(self.TAB_NAME, self.widget, self.preferences['pane'])
+		self.widget.show_all()
 
-	def toggle_fileview(self, enable=None):
-		self.toggle_action('toggle_fileview', active=enable)
+	@toggle_action(
+		_('Attachment Browser'), # T: Menu item
+		gtk.STOCK_DIRECTORY,
+		tooltip=_('Show Attachment Browser') # T: Toolbar item tooltip
+	)
+	def toggle_fileview(self, active):
+		# This toggle is called to focus on our widget
+		# but also after the fact when we detect focus changed
+		# so check state explicitly and don't do more than needed
+		visible, size, tab = self.window.get_pane_state(self.preferences['pane'])
 
-	def do_toggle_fileview(self, enable=None):
-		# TODO make this a generic "do_toggle_widget" ?
-		#~ print 'do_toggle_fileview', enable
-		if enable is None:
-			action = self.actiongroup.get_action('toggle_fileview')
-			enable = action.get_active()
-
-		if self._block_toggle:
-			self.statusbar_button.set_active(enable) # sync statusbar button
-			return
-
-		if enable:
-			self.ui.mainwindow.set_pane_state(
-				self.preferences['pane'], True,
-				activetab=self.TAB_NAME,
-				grab_focus=True)
+		if active:
+			if not (visible and tab == self.TAB_NAME):
+				self.window.set_pane_state(
+					self.preferences['pane'], True,
+					activetab=self.TAB_NAME,
+					grab_focus=True)
+			# else pass
 		else:
-			self.ui.mainwindow.set_pane_state(
-				self.preferences['pane'], False)
-
-		self.statusbar_button.set_active(enable) # sync statusbar button
+			if visible and tab == self.TAB_NAME:
+				self.window.set_pane_state(
+					self.preferences['pane'], False)
+			# else pass
 
 	def on_pane_state_changed(self, window, pane, visible, active):
 		if pane != self.preferences['pane']:
 			return
 
-		self._block_toggle = True
 		if visible and active == self.TAB_NAME:
 			self.toggle_fileview(True)
 			if not self.widget.get_active():
@@ -266,21 +350,26 @@ This plugin is still under development.
 		else:
 			self.toggle_fileview(False)
 			self.widget.set_active(False)
-		self._block_toggle = False
 
 	def on_open_page(self, ui, page, path):
 		self._disconnect_monitor()
 
-		self.widget.set_page(page)
+		self.widget.set_page(ui.notebook, page) # XXX
 		self._refresh_statusbar(page)
 
-		dir = self.ui.notebook.get_attachments_dir(page)
+		dir = self.window.ui.notebook.get_attachments_dir(page) # XXX -> page.get_attachemnts_dir()
 		id = dir.connect('changed', self.on_dir_changed)
 		self._monitor = (dir, id)
 
+	def _disconnect_monitor(self):
+		if self._monitor:
+			dir, id = self._monitor
+			dir.disconnect(id)
+			self._monitor = None
+
 	def on_dir_changed(self, *a):
 		logger.debug('Dir change detected: %s', a)
-		self._refresh_statusbar(self.ui.page)
+		self._refresh_statusbar(self.window.ui.page) # XXX
 		self.widget.refresh()
 
 	def _refresh_statusbar(self, page):
@@ -293,7 +382,7 @@ This plugin is still under development.
 		# Calculate independent from the widget
 		# (e.g. widget is not refreshed when hidden)
 		n = 0
-		dir = self.ui.notebook.get_attachments_dir(page)
+		dir = self.window.ui.notebook.get_attachments_dir(page) # XXX -> page.get_
 		from zim.fs import isdir
 		for name in dir.list():
 			# If dir is an attachment folder, sub-pages maybe filtered out already
@@ -304,41 +393,49 @@ This plugin is still under development.
 				n += 1
 		return n
 
-	def disconnect(self):
+	def destroy(self):
 		self._disconnect_monitor()
-		if self.ui.ui_type == 'gtk':
-			self.do_toggle_fileview(enable=False)
-			if self.statusbar_frame:
-				self.ui.mainwindow.statusbar.remove(self.statusbar_frame)
-		PluginClass.disconnect(self)
+		self.toggle_fileview(False)
+		if self.statusbar_frame:
+			self.window.statusbar.remove(self.statusbar_frame)
 
-	def _disconnect_monitor(self):
-		if self._monitor:
-			dir, id = self._monitor
-			dir.disconnect(id)
-			self._monitor = None
+		WindowExtension.destroy(self)
 
-	def do_preferences_changed(self):
-		if self.ui.ui_type == 'gtk':
-			try:
-				self.ui.mainwindow.remove(self.widget)
-			except ValueError:
-				pass
-			self.ui.mainwindow.add_tab(self.TAB_NAME, self.widget, self.preferences['pane'])
-			self.widget.show_all()
 
 
+class uistate_property(object):
+
+	# TODO add hook such that it will be initialized on init of owner obj
+
+	def __init__(self, key, *default):
+		self.key = key
+		self.default = default
+		self._initialized = False
+
+	def __get__(self, obj, klass):
+		if obj:
+			if not self._initialized:
+				obj.uistate.setdefault(self.key, *self.default)
+				self._initialized = True
+			return obj.uistate[self.key]
+
+	def __set__(self, obj, value):
+		obj.uistate[self.key] = value
+
 
 BASENAME_COL = 0
 PIXBUF_COL = 1
 
-
 class AttachmentBrowserPluginWidget(gtk.HBox, WindowSidePaneWidget):
 
-	def __init__(self, plugin, preferences):
+	icon_size = uistate_property('icon_size', DEFAULT_ICON_SIZE)
+
+	def __init__(self, extension, opener, preferences):
 		gtk.HBox.__init__(self)
-		self.plugin = plugin
-		self.ui = plugin.ui
+		self.page = None # XXX
+		self.extension = extension # XXX
+		self.opener = opener
+		self.uistate = extension.uistate
 		self.preferences = preferences
 		self.dir = None
 		self._active = True
@@ -353,7 +450,6 @@ class AttachmentBrowserPluginWidget(gtk.HBox, WindowSidePaneWidget):
 		self.fileview = gtk.IconView(self.store)
 		self.fileview.set_text_column(BASENAME_COL)
 		self.fileview.set_pixbuf_column(PIXBUF_COL)
-		self.fileview.set_item_width(ICON_SIZE * 2) # Force wrapping text
 
 		self.fileview.enable_model_drag_source(
 			gtk.gdk.BUTTON1_MASK,
@@ -378,8 +474,16 @@ class AttachmentBrowserPluginWidget(gtk.HBox, WindowSidePaneWidget):
 		refresh_button.connect('clicked', lambda o: self.on_refresh_button())
 		self.buttonbox.pack_start(refresh_button, False)
 
+		zoomin = IconButton(gtk.STOCK_ZOOM_IN, relief=False)
+		zoomout = IconButton(gtk.STOCK_ZOOM_OUT, relief=False)
+		zoomin.connect('clicked', lambda o: self.zoom_in())
+		zoomout.connect('clicked', lambda o: self.zoom_out())
+		self.buttonbox.pack_end(zoomout, False)
+		self.buttonbox.pack_end(zoomin, False)
 		self.fileview.connect('button-press-event', self.on_button_press_event)
 		self.fileview.connect('item-activated', self.on_item_activated)
+		self.zoomin_button = zoomin
+		self.zoomout_button = zoomout
 
 		if gtk.gtk_version >= (2, 12):
 			# custom tooltip
@@ -411,8 +515,9 @@ class AttachmentBrowserPluginWidget(gtk.HBox, WindowSidePaneWidget):
 					self.buttonbox.remove(widget)
 		return True
 
-	def set_page(self, page):
-		dir = self.ui.notebook.get_attachments_dir(page)
+	def set_page(self, notebook, page):
+		self.page = page # XXX
+		dir = notebook.get_attachments_dir(page) # XXX page.get_attachments_dir
 		self.set_folder(dir)
 
 	def set_folder(self, dir):
@@ -430,21 +535,36 @@ class AttachmentBrowserPluginWidget(gtk.HBox, WindowSidePaneWidget):
 
 	def on_open_folder(self, o):
 		# Callback for the "open folder" button
-		self.ui.open_attachments_folder()
+		self.opener.open_dir(self.dir)
 		self._update_state()
 
 	def on_refresh_button(self):
 		self.refresh()
-		self.plugin._refresh_statusbar(self.ui.page) # bit of a HACK to get the page here
+		self.extension._refresh_statusbar(self.page)
+			# XXX FIXME communicate to extension by signal
+
+	def zoom_in(self):
+		self.icon_size = min((self.icon_size * 2, THUMB_SIZE_LARGE)) # 16 > 32 > 64 > 128 > 256
+		self.refresh()
+
+	def zoom_out(self):
+		self.icon_size = max((self.icon_size/2, MIN_ICON_SIZE)) # 16 < 32 < 64 < 128 < 256
+		self.refresh()
 
 	def refresh(self):
 		if not self._active:
 			return # avoid unnecessary work
 
+		# Clear data
 		self.store.clear()
 		self.thumbman.clear_async_queue()
 		self._update_state()
 
+		# Update once, re-use below
+		self._file_icon = render_file_icon(self, min((self.icon_size, THUMB_SIZE_NORMAL)))
+		self._file_icon_tooltip = render_file_icon(self, THUMB_SIZE_NORMAL)
+
+		# Add files
 		for name in self.dir.list():
 			# If dir is an attachment folder, sub-pages maybe filtered out already
 			file = self.dir.file(name)
@@ -453,6 +573,41 @@ class AttachmentBrowserPluginWidget(gtk.HBox, WindowSidePaneWidget):
 			else:
 				self._add_file(file)
 
+		# Update column size etc.
+		if self.icon_size < 64:
+			# Would like to switch to by-column layout here, but looks
+			# like gtk.IconView does not support that
+			self.fileview.set_orientation(gtk.ORIENTATION_HORIZONTAL)
+			size = self._get_max_width()
+			if size > 0:
+				if self.icon_size > 16:
+					size = int((size+1) / 2) # Wrap over 2 rows
+				# Else no wrap
+				size += self.icon_size
+				self.fileview.set_item_width(size)
+			self.fileview.set_row_spacing(0)
+			self.fileview.set_column_spacing(0)
+		else:
+			self.fileview.set_orientation(gtk.ORIENTATION_VERTICAL)
+			size = max((self.icon_size + 12, 96))
+			self.fileview.set_item_width(size)
+				# Set item width to force wrapping text for long items
+				# Set to icon size + some space for padding etc.
+			self.fileview.set_row_spacing(3)
+			self.fileview.set_column_spacing(3)
+
+	def _get_max_width(self):
+		import pango
+		layout = self.fileview.create_pango_layout('')
+		model = self.fileview.get_model()
+		l = 0
+		for r in model:
+			l = max((l, len(r[BASENAME_COL])))
+			#layout.set_text(r[BASENAME_COL])
+			#l = max((l, layout.get_pixel_size()[0]))
+		return l * 10 # XXX Rough estimate with 10px per char...
+
+
 	def _update_state(self):
 		# Here we set color like senstive or insensitive widget without
 		# really making the widget insensitive - reason is to allow
@@ -461,23 +616,35 @@ class AttachmentBrowserPluginWidget(gtk.HBox, WindowSidePaneWidget):
 		if self.dir is None or not self.dir.exists():
 			self.fileview.modify_base(
 				gtk.STATE_NORMAL, self._insenstive_color)
-			return # Show empty view
+
+			self.zoomin_button.set_sensitive(False)
+			self.zoomout_button.set_sensitive(False)
 		else:
 			self.fileview.modify_base(
 				gtk.STATE_NORMAL, self._senstive_color)
 
+			self.zoomin_button.set_sensitive(self.icon_size < THUMB_SIZE_LARGE)
+			self.zoomout_button.set_sensitive(self.icon_size > MIN_ICON_SIZE)
+
+
 	def _add_file(self, file):
-		pixbuf = self.thumbman.get_thumbnail_async(file, ICON_SIZE)
+		if self.icon_size >= DEFAULT_ICON_SIZE:
+			# Only use thumbnails if icons sufficiently large to show them
+			pixbuf = self.thumbman.get_thumbnail_async(file, self.icon_size)
+		else:
+			pixbuf = None
+
 		if pixbuf is None:
 			# Set generic icon first - maybe thumbnail follows later, maybe not
-			pixbuf = get_mime_icon(file, ICON_SIZE) \
-				or self.render_icon(gtk.STOCK_FILE, ICON_SIZE)
+			# Icon size for mime icons is limitted, avoid huge icons
+			pixbuf = get_mime_icon(file, min((self.icon_size, THUMB_SIZE_NORMAL))) \
+				or self._file_icon
 
 		self.store.append((file.basename, pixbuf)) # BASENAME_COL, PIXBUF_COL
 
 	def on_thumbnail_ready(self, o, file, size, pixbuf):
 		#~ print "GOT THUMB:", file, size, pixbuf
-		if size != ICON_SIZE or file.dir != self.dir:
+		if size != self.icon_size or file.dir != self.dir:
 			return
 
 		basename = file.basename
@@ -490,7 +657,7 @@ class AttachmentBrowserPluginWidget(gtk.HBox, WindowSidePaneWidget):
 	def on_item_activated(self, iconview, path):
 		iter = self.store.get_iter(path)
 		file = self.dir.file(self.store[iter][BASENAME_COL])
-		self.ui.open_file(file)
+		self.opener.open_file(file)
 
 	def on_button_press_event(self, iconview, event):
 		# print 'on_button_press_event'
@@ -516,11 +683,11 @@ class AttachmentBrowserPluginWidget(gtk.HBox, WindowSidePaneWidget):
 		item = gtk.MenuItem(_('Open With...')) # T: menu item
 		menu.prepend(item)
 
-		submenu = OpenWithMenu(self.ui, file)
+		submenu = OpenWithMenu(self.extension.window, file) # XXX any widget should do to find window
 		item.set_submenu(submenu)
 
 		item = gtk.MenuItem(_('_Open')) # T: menu item to open file or folder
-		item.connect('activate', lambda o: self.ui.open_file(file))
+		item.connect('activate', lambda o: self.opener.open_file(file))
 		menu.prepend(item)
 
 		menu.show_all()
@@ -541,18 +708,25 @@ class AttachmentBrowserPluginWidget(gtk.HBox, WindowSidePaneWidget):
 			mdate = _('Unknown') # T: unspecified value for file modification time
 		size = format_file_size(file.size())
 
-		pixbuf = self.thumbman.get_thumbnail(file, PREVIEW_SIZE)
+		pixbuf = self.thumbman.get_thumbnail(file, THUMB_SIZE_LARGE)
 		if not pixbuf:
-			pixbuf = model[iter][PIXBUF_COL]
+			# No thumbnail, use icon, but use it at normal size
+			pixbuf = get_mime_icon(file, THUMB_SIZE_NORMAL) \
+				or self._file_icon_tooltip
 
-		# TODO stat file for size and m_time
+		mtype = file.get_mimetype()
+		mtype_desc = get_mime_description(mtype)
+		if mtype_desc:
+			mtype_desc = mtype_desc + " (%s)" % mtype # E.g. "PDF document (application/pdf)"
 
 		f_label = _('Name') # T: label for file name
+		t_label = _('Type') # T: label for file type
 		s_label = _('Size') # T: label for file size
 		m_label = _('Modified') # T: label for file modification date
 		tooltip.set_markup(
-			"%s\n\n<b>%s:</b> %s\n<b>%s:</b>\n%s" % (
+			"%s\n\n<b>%s:</b> %s\n<b>%s:</b> %s\n<b>%s:</b>\n%s" % (
 				name,
+				t_label, mtype_desc or mtype,
 				s_label, size,
 				m_label, mdate,
 			))
